/*
 * Copyright (c) 2013-2023 NIBIO.
 *
 * This file is part of VIPSWeb 
 * (see https://gitlab.nibio.no/VIPS/VIPSWeb).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 */

/**
*
*/

/**
 * Setting some global options on Higcharts, like language configs
 */
if(typeof Highcharts !== "undefined")
{
	Highcharts.setOptions(settings.highchartsGlobalOptions);
}

var Organism = function(oJSON)
{
	if(oJSON != null)
	{
		this.organismId = oJSON.organismId;
		this.latinName = oJSON.latinName;
		this.tradeName = oJSON.tradeName;
		this.organismLocaleSet = oJSON.organismLocaleSet;
	}
};

/**
 * TODO: This should be in the organisms app?
 * Returns 1) the local name, or 2) English name or 3) Latin name
 */
Organism.prototype.getLocalName = function(languageCode)
{
	var retVal;
	for(var i in this.organismLocaleSet)
	{
		var organismLocale = this.organismLocaleSet[i];
		if(retVal == null && organismLocale.organismLocalePK.locale == "en")
		{
			retVal = organismLocale.localName;
		}
		if(organismLocale.organismLocalePK.locale == languageCode)
		{
			return organismLocale.localName;
		}
	}
	return retVal != null ? retVal : this.latinName;
};

var PointOfInterest = function(pJSON)
{
	this.pointOfInterestId = pJSON != null ? pJSON.pointOfInterestId : -1;
	this.name = pJSON != null ? pJSON.name : "";
};

// Defining the forecast configuration object
var ForecastConfiguration = function(fJSON)
{
	if(fJSON != null)
	{
		this.forecastConfigurationId = fJSON.forecastConfigurationId;
		this.modelId = fJSON.modelId;
		this.dateStart = moment(fJSON.dateStart);
		this.dateEnd = moment(fJSON.dateEnd);
		this.cropOrganism = new Organism(fJSON.cropOrganismId);
		this.pestOrganism = new Organism(fJSON.pestOrganismId);
		this.location = new PointOfInterest(fJSON.locationPointOfInterestId);
		this.weatherStation = new PointOfInterest(fJSON.weatherStationPointOfInterestId);
	}
};

ForecastConfiguration.prototype.compareTo = function(a,b)
{
	if(a.dateStart.isBefore(b.dateStart)) return -1;
	if(a.dateStart.isSame(b.dateStart)) return 0;
	if(a.dateStart.isAfter(b.dateStart)) return 1;
};

ForecastConfiguration.prototype.getInstanceListFromJSON = function(JSON)
{
	var retVal = [];
	for(var i in JSON)
	{
		retVal.push(new ForecastConfiguration(JSON[i]))
	}
	return retVal;
};

function getModelLocalName(modelsLocalNames, modelId)
{
	for(var i in modelsLocalNames)
	{
		var modelLocalName = modelsLocalNames[i];
		if(modelLocalName.modelId == modelId)
		{
			return modelLocalName.modelName;
		}
	}
	return modelId;
};

/**
 * Renders a forecast chart 
 * @param DOMContainerId String with id of HTML container to render to
 * @param chartTitle String, title of chart
 * @param warningStatusPlotBandData Array of plot band data. See http://www.highcharts.com/docs/chart-concepts/plot-bands-and-plot-lines 
 *  Sample: 
 * <pre>
 * [
 * 		{'color': '#F0564D', 'to': 1428942600000.0, 'from': 1428877800000.0}, 
 * 		{'color': '#6FC49A', 'to': 1429068600000.0, 'from': 1428942600000.0}, 
 * 		{'color': '#FEDA4D', 'to': 1429075800000.0, 'from': 1429068600000.0}, 
 * 		{'color': '#F0564D', 'to': 1429234200000.0, 'from': 1429075800000.0}
 * ]
 * </pre>
 * @param data Highcharts formatted data in series. See http://www.highcharts.com/docs/chart-concepts/series 
 */
function renderForecastChart(DOMContainerId, chartTitle, warningStatusPlotBandData, data, customOptions)
{
	// Avoid nullpointers
	if(customOptions == undefined || customOptions == null)
	{
		customOptions = {};
	}
	var options = {
			
			exporting: customOptions["exporting"] != null ? customOptions["exporting"] : {},
	    	title: {
	    		text: chartTitle,
	    	},
	        chart: {
	        	backgroundColor: customOptions["backgroundColor"] != null ? customOptions["backgroundColor"] : "white",
	            renderTo: DOMContainerId,
	            type: 'spline',
	            zoomType: "x"
	        },
	        plotOptions: {
	            series: {
	                allowPointSelect: false,
	                animation: false
	            },
	            spline: {
	            	marker: {
	            		enabled:false
	            	},
	            },
	            scatter: {
	            	marker: {
	            		enabled:true
	            	},
	                tooltip: {
	                    pointFormat: '{point.x:%Y-%m-%d}: {point.y}'
	                }
	            }
	        },
	        xAxis: {
	            type: 'datetime',
	            maxZoom: 4 * 24 * 3600000, // four days
	            dateTimeLabelFormats: { // don't display the dummy year
	                month: '%e. %b',
	                year: '%b'
	           },
	           plotBands: warningStatusPlotBandData, // This is where we put warning status background bands
	           plotLines: [{
	           		color: "white",
	           		width: 3,
	                value: getSystemTime(),
	                dashStyle: 'shortdot',
	                zIndex: 100
	           }]
	        },
	        yAxis: [],
	        series: []
	    };

	    for(index in data.yAxis)
		{
			options.yAxis[index] = createYAxisConf(data.yAxis[index], index == 0);
		}
		
		var warningNS = "WARNING";
		var warningPropNames = ["NOFORECAST","MISSINGDATA","NORISK","MEDIUMRISK","HIGHRISK"];
		for(propName in data.series){
			if(propName.indexOf(warningNS) != 0)
			{
				options.series[options.series.length] = data.series[propName];
			}
		}
		//console.log("Before creating chart " + (new Date().getTime() % 10000000));
	    var chart = new Highcharts.Chart(options);
}

/**
 * Based on a dict with axisInfo, creates a configuration for an axis
 */
function createYAxisConf(axisInfo, isFirst)
{
	return {
		gridLineWidth: isFirst ? 1 : 0,
		opposite: axisInfo.opposite,
		title: {
			text: axisInfo.hideAxis ? "" : axisInfo.title,
			style: {
				color: axisInfo.color
			}
		},
		labels: {
			formatter: function(){
				return axisInfo.hideAxis ? "" : this.value + axisInfo.abbr;
			},
			style: {
				color: axisInfo.color
			}
		},
		max: axisInfo.max,
		min: axisInfo.min,
		allowDecimals: axisInfo.allowDecimals != null ? axisInfo.allowDecimals : true,
		ceiling: axisInfo.ceiling,
		floor: axisInfo.floor
	};
}

/**
 * 
 */
function changeLatestDays(theForm){
	window.location.href="/forecasts/" + theForm.forecastConfigurationId.value + "/" + theForm.latestDays.value;
}

/**
 * Returns an array with plot band data to be used in HighCharts
 * @param results
 */
function getWarningStatusPlotBandData(forecastResults)
{
	//console.info(forecastResults);
	var warningStatuses = {
            0: {name:"No forecast available", color:"#C9C9C9"},
            1: {name:"Missing data", color:"#4DA0FE"},
            2: {name:"No risk", color:"#6FC49A"},
            3: {name:"Medium risk",color:"#FEDA4D"},
            4: {name:"High risk",color:"#F0564D"}
    };
	var plotBands = [];
	var previousForecastResult = null;
	var plotBand = null;
	var bandOffset = 0;
	if (forecastResults != null && forecastResults.length >=2)
	{
		// Replace with Moment.js
		var start = getUnixTimestampFromJSON(forecastResults[0]["validTimeStart"]);
		var end = getUnixTimestampFromJSON(forecastResults[1]["validTimeStart"]);
		
		bandOffset = (end - start) / 2;
		//console.log("bandOffset=" + bandOffset);
	}
	for(var i in forecastResults)
	{
		var forecastResult = forecastResults[i];
		if(previousForecastResult == null || previousForecastResult["warningStatus"] != forecastResult["warningStatus"])
		{
			// Add the previous plot band
			if(plotBand != null)
			{
				// Moment.js needed!
			    plotBand["to"] = getUnixTimestampFromJSON(forecastResult["validTimeStart"]) - bandOffset;
			    plotBands.push(plotBand);
			}
			plotBand = {
		         "color": warningStatuses[forecastResult["warningStatus"]].color,
		         // Moment.js!!
		         "from" : getUnixTimestampFromJSON(forecastResult["validTimeStart"]) - bandOffset
	        };
		}
		previousForecastResult = forecastResult;
	}
	
	// We probably have a rouge plot band that needs to be finished and appended
	if(plotBand != null && plotBand["to"] == null)
	{
		// Moment.js!!!
		plotBand.to = getUnixTimestampFromJSON(forecastResults[forecastResults.length -1]["validTimeStart"]) + bandOffset;
		plotBands.push(plotBand);
	}
	
	return plotBands;
}

function getHighChartsSeries(forecastResults, yAxisList, parameterList)
{
	var chartSeries = {
			series: {},
			yAxis: yAxisList
	};
	
	for(var i in parameterList)
	{
		var parameter = parameterList[i];
		chartSeries.series[parameter.key] = {
				data: [],
				type: parameter.type,
				name: parameter.name,
				showInLegend: (parameter.showInLegend === undefined || parameter.showInLegend === null || parameter.showInLegend === true),
				color: parameter.color,
				yAxis: parameter.yAxis,
				lineWidth: parameter.lineWidth !== undefined ?  parameter.lineWidth: 2
		};
	}
	
	for(var i in forecastResults)
	{
		var forecastResult = forecastResults[i];
		var allValues = getJSON(forecastResult["allValues"]);
		for(var j in parameterList)
		{
			var parameter = parameterList[j];
			var value = parseFloat(allValues[parameter.key]);
			chartSeries.series[parameter.key].data.push([getUnixTimestampFromJSON(forecastResult.validTimeStart),value]);
		}
	}
	
	return chartSeries;
}